@@ -81,6 +81,7 @@ public:
     const std::unordered_map<std::string, std::string>& params,
     const std::string& name, bool* out);
 
+  static void KV2Map(std::unordered_map<std::string, std::string>& params, const char* kv);
   static std::unordered_map<std::string, std::string> Str2Map(const char* parameters);
 };
 
@@ -121,7 +122,7 @@ public:
   bool is_predict_contrib = false;
   bool is_predict_raw_score = false;
   int min_data_in_leaf = 20;
-  int min_data_in_bin = 5;
+  int min_data_in_bin = 3;
   double max_conflict_rate = 0.0f;
   bool enable_bundle = true;
   bool has_header = false;
@@ -224,12 +225,11 @@ public:
   int gpu_device_id = -1;
   /*! \brief Set to true to use double precision math on GPU (default using single precision) */
   bool gpu_use_dp = false;
-  int max_cat_group = 64;
-  int min_data_per_group = 10;
-  int max_cat_threshold = 256;
-  double cat_smooth_ratio = 0.01;
-  double min_cat_smooth = 5;
-  double max_cat_smooth = 100;
+  int min_data_per_group = 100;
+  int max_cat_threshold = 32;
+  double cat_l2 = 10;
+  double cat_smooth = 10;
+  int max_cat_to_onehot = 4;
   LIGHTGBM_EXPORT void Set(const std::unordered_map<std::string, std::string>& params) override;
 };
 
@@ -270,6 +270,7 @@ public:
   int local_listen_port = 12400;
   int time_out = 120;  // in minutes
   std::string machine_list_filename = "";
+  std::string machines = "";
   LIGHTGBM_EXPORT void Set(const std::unordered_map<std::string, std::string>& params) override;
 };
 
@@ -437,7 +438,9 @@ struct ParameterAlias {
       { "num_classes", "num_class" },
       { "unbalanced_sets", "is_unbalance" },
       { "bagging_fraction_seed", "bagging_seed" },
-      { "num_boost_round", "num_iterations" }
+      { "num_boost_round", "num_iterations" },
+      { "workers", "machines" },
+      { "nodes", "machines" },
     });
     const std::unordered_set<std::string> parameter_set({
       "config", "config_file", "task", "device",
@@ -467,21 +470,44 @@ struct ParameterAlias {
       "feature_fraction_seed", "enable_bundle", "data_filename", "valid_data_filenames",
       "snapshot_freq", "verbosity", "sparse_threshold", "enable_load_from_binary_file",
       "max_conflict_rate", "poisson_max_delta_step", "gaussian_eta",
-      "histogram_pool_size", "output_freq", "is_provide_training_metric", "machine_list_filename",
+      "histogram_pool_size", "output_freq", "is_provide_training_metric", "machine_list_filename", "machines",
       "zero_as_missing", "init_score_file", "valid_init_score_file", "is_predict_contrib",
-      "max_cat_threshold", "max_cat_group", "cat_smooth_ratio", "min_cat_smooth", "max_cat_smooth", "min_data_per_group"
+      "max_cat_threshold",  "cat_smooth", "min_data_per_group", "cat_l2", "max_cat_to_onehot"
     });
     std::unordered_map<std::string, std::string> tmp_map;
     for (const auto& pair : *params) {
-      if (alias_table.count(pair.first) > 0) {
-        tmp_map[alias_table.at(pair.first)] = pair.second;
-      } else if (parameter_set.count(pair.first) == 0) {
-        Log::Fatal("Unknown parameter: %s", pair.first.c_str());
+      auto alias = alias_table.find(pair.first);
+      if (alias != alias_table.end()) { // found alias
+        auto alias_set = tmp_map.find(alias->second); 
+        if (alias_set != tmp_map.end()) { // alias already set
+          // set priority by length & alphabetically to ensure reproducible behavior
+          if (alias_set->second.size() < pair.first.size() ||
+            (alias_set->second.size() == pair.first.size() && alias_set->second < pair.first)) {
+            Log::Warning("%s is set with %s=%s, %s=%s will be ignored. Current value: %s=%s.",
+              alias->second.c_str(), alias_set->second.c_str(), params->at(alias_set->second).c_str(),
+              pair.first.c_str(), pair.second.c_str(), alias->second.c_str(), params->at(alias_set->second).c_str());
+          } else {
+            Log::Warning("%s is set with %s=%s, will be overrided by %s=%s. Current value: %s=%s.",
+              alias->second.c_str(), alias_set->second.c_str(), params->at(alias_set->second).c_str(),
+              pair.first.c_str(), pair.second.c_str(), alias->second.c_str(), pair.second.c_str());
+            tmp_map[alias->second] = pair.first;
+          }
+        } else { // alias not set
+          tmp_map.emplace(alias->second, pair.first);
+        }
+      } else if (parameter_set.find(pair.first) == parameter_set.end()) {
+        Log::Warning("Unknown parameter: %s", pair.first.c_str());
       }
     }
     for (const auto& pair : tmp_map) {
-      if (params->count(pair.first) == 0) {
-        params->insert(std::make_pair(pair.first, pair.second));
+      auto alias = params->find(pair.first);
+      if (alias == params->end()) { // not find
+        params->emplace(pair.first, params->at(pair.second));
+        params->erase(pair.second);
+      } else {
+        Log::Warning("%s is set=%s, %s=%s will be ignored. Current value: %s=%s.", 
+          pair.first.c_str(), alias->second.c_str(), pair.second.c_str(), params->at(pair.second).c_str(),
+          pair.first.c_str(), alias->second.c_str());
       }
     }
   }
