@@ -12,21 +12,31 @@
 
 namespace LightGBM {
 
+void ConfigBase::KV2Map(std::unordered_map<std::string, std::string>& params, const char* kv) {
+  std::vector<std::string> tmp_strs = Common::Split(kv, '=');
+  if (tmp_strs.size() == 2) {
+    std::string key = Common::RemoveQuotationSymbol(Common::Trim(tmp_strs[0]));
+    std::string value = Common::RemoveQuotationSymbol(Common::Trim(tmp_strs[1]));
+    if (key.size() > 0) {
+      auto value_search = params.find(key);
+      if (value_search == params.end()) { // not set
+        params.emplace(key, value);
+      } else {
+        Log::Warning("%s is set=%s, %s=%s will be ignored. Current value: %s=%s.",
+          key.c_str(), value_search->second.c_str(), key.c_str(), value.c_str(),
+          key.c_str(), value_search->second.c_str());
+      }
+    }
+  } else {
+    Log::Warning("Unknown parameter %s", kv);
+  }
+}
+
 std::unordered_map<std::string, std::string> ConfigBase::Str2Map(const char* parameters) {
   std::unordered_map<std::string, std::string> params;
   auto args = Common::Split(parameters, " \t\n\r");
   for (auto arg : args) {
-    std::vector<std::string> tmp_strs = Common::Split(arg.c_str(), '=');
-    if (tmp_strs.size() == 2) {
-      std::string key = Common::RemoveQuotationSymbol(Common::Trim(tmp_strs[0]));
-      std::string value = Common::RemoveQuotationSymbol(Common::Trim(tmp_strs[1]));
-      if (key.size() <= 0) {
-        continue;
-      }
-      params[key] = value;
-    } else if (Common::Trim(arg).size() > 0) {
-      Log::Warning("Unknown parameter %s", arg.c_str());
-    }
+    KV2Map(params, Common::Trim(arg).c_str());
   }
   ParameterAlias::KeyAliasTransform(&params);
   return params;
@@ -370,18 +380,16 @@ void TreeConfig::Set(const std::unordered_map<std::string, std::string>& params)
   GetInt(params, "gpu_platform_id", &gpu_platform_id);
   GetInt(params, "gpu_device_id", &gpu_device_id);
   GetBool(params, "gpu_use_dp", &gpu_use_dp);
-  GetInt(params, "max_cat_group", &max_cat_group);
   GetInt(params, "max_cat_threshold", &max_cat_threshold);
-  GetDouble(params, "cat_smooth_ratio", &cat_smooth_ratio);
-  GetDouble(params, "min_cat_smooth", &min_cat_smooth);
-  GetDouble(params, "max_cat_smooth", &max_cat_smooth);
+  GetDouble(params, "cat_l2", &cat_l2);
+  GetDouble(params, "cat_smooth", &cat_smooth);
   GetInt(params, "min_data_per_group", &min_data_per_group);
-  CHECK(max_cat_group > 1);
+  GetInt(params, "max_cat_to_onehot", &max_cat_to_onehot);
   CHECK(max_cat_threshold > 0);
-  CHECK(cat_smooth_ratio >= 0);
-  CHECK(min_cat_smooth >= 1);
-  CHECK(max_cat_smooth > min_cat_smooth);
+  CHECK(cat_l2 >= 0.0f);
+  CHECK(cat_smooth >= 1);
   CHECK(min_data_per_group > 0);
+  CHECK(max_cat_to_onehot > 0);
 }
 
 void BoostingConfig::Set(const std::unordered_map<std::string, std::string>& params) {
@@ -425,6 +433,7 @@ void NetworkConfig::Set(const std::unordered_map<std::string, std::string>& para
   GetInt(params, "time_out", &time_out);
   CHECK(time_out > 0);
   GetString(params, "machine_list_file", &machine_list_filename);
+  GetString(params, "machines", &machines);
 }
 
 }  // namespace LightGBM
