@@ -11,7 +11,7 @@ import numpy as np
 
 from . import callback
 from .basic import Booster, Dataset, LightGBMError, _InnerPredictor
-from .compat import (SKLEARN_INSTALLED, LGBMGroupKFold, LGBMStratifiedKFold,
+from .compat import (SKLEARN_INSTALLED, _LGBMGroupKFold, _LGBMStratifiedKFold,
                      integer_types, range_, string_type)
 
 
@@ -22,8 +22,7 @@ def train(params, train_set, num_boost_round=100,
           early_stopping_rounds=None, evals_result=None,
           verbose_eval=True, learning_rates=None,
           keep_training_booster=False, callbacks=None):
-    """
-    Train with given parameters.
+    """Perform the training with given parameters.
 
     Parameters
     ----------
@@ -31,68 +30,67 @@ def train(params, train_set, num_boost_round=100,
         Parameters for training.
     train_set : Dataset
         Data to be trained.
-    num_boost_round: int
+    num_boost_round: int, optional (default=100)
         Number of boosting iterations.
-    valid_sets: list of Datasets
-        List of data to be evaluated during training
-    valid_names: list of string
-        Names of valid_sets
-    fobj : function
+    valid_sets: list of Datasets or None, optional (default=None)
+        List of data to be evaluated during training.
+    valid_names: list of string or None, optional (default=None)
+        Names of ``valid_sets``.
+    fobj : callable or None, optional (default=None)
         Customized objective function.
-    feval : function
+    feval : callable or None, optional (default=None)
         Customized evaluation function.
-        Note: should return (eval_name, eval_result, is_higher_better) of list of this
-    init_model : file name of lightgbm model or 'Booster' instance
-        model used for continued train
-    feature_name : list of str, or 'auto'
-        Feature names
-        If 'auto' and data is pandas DataFrame, use data columns name
-    categorical_feature : list of str or int, or 'auto'
-        Categorical features,
-        type int represents index,
-        type str represents feature names (need to specify feature_name as well)
-        If 'auto' and data is pandas DataFrame, use pandas categorical columns
-    early_stopping_rounds: int
-        Activates early stopping.
-        Requires at least one validation data and one metric
-        If there's more than one, will check all of them
-        Returns the model with (best_iter + early_stopping_rounds)
-        If early stopping occurs, the model will add 'best_iteration' field
-    evals_result: dict or None
-        This dictionary used to store all evaluation results of all the items in valid_sets.
-        Example: with a valid_sets containing [valid_set, train_set]
-                 and valid_names containing ['eval', 'train']
-                 and a paramater containing ('metric':'logloss')
-        Returns: {'train': {'logloss': ['0.48253', '0.35953', ...]},
-                  'eval': {'logloss': ['0.480385', '0.357756', ...]}}
-        passed with None means no using this function
-    verbose_eval : bool or int
-        Requires at least one item in evals.
-        If `verbose_eval` is True,
-            the eval metric on the valid set is printed at each boosting stage.
-        If `verbose_eval` is int,
-            the eval metric on the valid set is printed at every `verbose_eval` boosting stage.
-        The last boosting stage
-            or the boosting stage found by using `early_stopping_rounds` is also printed.
-        Example: with verbose_eval=4 and at least one item in evals,
-            an evaluation metric is printed every 4 (instead of 1) boosting stages.
-    learning_rates: list or function
-        List of learning rate for each boosting round
-        or a customized function that calculates learning_rate
-        in terms of current number of round (e.g. yields learning rate decay)
-        - list l: learning_rate = l[current_round]
-        - function f: learning_rate = f(current_round)
-    keep_training_booster : boolean
-        Whether the return booster will be used to keep training.
-        If false, will convert into _InnerPredictor before return.
-        You can still use _InnerPredictor as init_model for future continue training.
-    callbacks : list of callback functions
+        Note: should return (eval_name, eval_result, is_higher_better) or list of such tuples.
+    init_model : string or None, optional (default=None)
+        Filename of LightGBM model or Booster instance used for continue training.
+    feature_name : list of strings or 'auto', optional (default="auto")
+        Feature names.
+        If 'auto' and data is pandas DataFrame, data columns names are used.
+    categorical_feature : list of strings or int, or 'auto', optional (default="auto")
+        Categorical features.
+        If list of int, interpreted as indices.
+        If list of strings, interpreted as feature names (need to specify ``feature_name`` as well).
+        If 'auto' and data is pandas DataFrame, pandas categorical columns are used.
+    early_stopping_rounds: int or None, optional (default=None)
+        Activates early stopping. The model will train until the validation score stops improving.
+        Requires at least one validation data and one metric. If there's more than one, will check all of them.
+        If early stopping occurs, the model will add ``best_iteration`` field.
+    evals_result: dict or None, optional (default=None)
+        This dictionary used to store all evaluation results of all the items in ``valid_sets``.
+
+        Example
+        -------
+        With a ``valid_sets`` = [valid_set, train_set],
+        ``valid_names`` = ['eval', 'train']
+        and a ``params`` = ('metric':'logloss')
+        returns: {'train': {'logloss': ['0.48253', '0.35953', ...]},
+        'eval': {'logloss': ['0.480385', '0.357756', ...]}}.
+    verbose_eval : bool or int, optional (default=True)
+        Requires at least one validation data.
+        If True, the eval metric on the valid set is printed at each boosting stage.
+        If int, the eval metric on the valid set is printed at every ``verbose_eval`` boosting stage.
+        The last boosting stage or the boosting stage found by using ``early_stopping_rounds`` is also printed.
+
+        Example
+        -------
+        With ``verbose_eval`` = 4 and at least one item in evals,
+        an evaluation metric is printed every 4 (instead of 1) boosting stages.
+    learning_rates: list, callable or None, optional (default=None)
+        List of learning rates for each boosting round
+        or a customized function that calculates ``learning_rate``
+        in terms of current number of round (e.g. yields learning rate decay).
+    keep_training_booster : bool, optional (default=False)
+        Whether the returned Booster will be used to keep training.
+        If False, the returned value will be converted into _InnerPredictor before returning.
+        You can still use _InnerPredictor as ``init_model`` for future continue training.
+    callbacks : list of callables or None, optional (default=None)
         List of callback functions that are applied at each iteration.
-        See Callbacks in Python-API.md for more information.
+        See Callbacks in Python API for more information.
 
     Returns
     -------
-    booster : a trained booster model
+    booster : Booster
+        The trained Booster model.
     """
     """create predictor first"""
     for alias in ["num_boost_round", "num_iterations", "num_iteration", "num_tree", "num_trees", "num_round", "num_rounds"]:
@@ -264,12 +262,12 @@ def _make_n_folds(full_data, folds, nfold, params, seed, fpreproc=None, stratifi
             # lambdarank task, split according to groups
             group_info = full_data.get_group().astype(int)
             flatted_group = np.repeat(range(len(group_info)), repeats=group_info)
-            group_kfold = LGBMGroupKFold(n_splits=nfold)
+            group_kfold = _LGBMGroupKFold(n_splits=nfold)
             folds = group_kfold.split(X=np.zeros(num_data), groups=flatted_group)
         elif stratified:
             if not SKLEARN_INSTALLED:
                 raise LightGBMError('Scikit-learn is required for stratified cv.')
-            skf = LGBMStratifiedKFold(n_splits=nfold, shuffle=shuffle, random_state=seed)
+            skf = _LGBMStratifiedKFold(n_splits=nfold, shuffle=shuffle, random_state=seed)
             folds = skf.split(X=np.zeros(num_data), y=full_data.get_label())
         else:
             if shuffle:
@@ -316,68 +314,71 @@ def cv(params, train_set, num_boost_round=10,
        early_stopping_rounds=None, fpreproc=None,
        verbose_eval=None, show_stdv=True, seed=0,
        callbacks=None):
-    """
-    Cross-validation with given paramaters.
+    """Perform the cross-validation with given paramaters.
 
     Parameters
     ----------
     params : dict
-        Booster params.
+        Parameters for Booster.
     train_set : Dataset
-        Data to be trained.
-    num_boost_round : int
+        Data to be trained on.
+    num_boost_round : int, optional (default=10)
         Number of boosting iterations.
-    folds : a generator or iterator of (train_idx, test_idx) tuples
-        The train indices and test indices for each folds.
+    folds : a generator or iterator of (train_idx, test_idx) tuples or None, optional (default=None)
+        The train and test indices for the each fold.
         This argument has highest priority over other data split arguments.
-    nfold : int
+    nfold : int, optional (default=5)
         Number of folds in CV.
-    stratified : bool
-        Perform stratified sampling.
-    shuffle: bool
-        Whether shuffle before split data
-    metrics : string or list of strings
-        Evaluation metrics to be watched in CV.
-        If `metrics` is not None, the metric in `params` will be overridden.
-    fobj : function
+    stratified : bool, optional (default=True)
+        Whether to perform stratified sampling.
+    shuffle: bool, optional (default=True)
+        Whether to shuffle before splitting data.
+    metrics : string, list of strings or None, optional (default=None)
+        Evaluation metrics to be monitored while CV.
+        If not None, the metric in ``params`` will be overridden.
+    fobj : callable or None, optional (default=None)
         Custom objective function.
-    feval : function
+    feval : callable or None, optional (default=None)
         Custom evaluation function.
-    init_model : file name of lightgbm model or 'Booster' instance
-        model used for continued train
-    feature_name : list of str, or 'auto'
-        Feature names
-        If 'auto' and data is pandas DataFrame, use data columns name
-    categorical_feature : list of str or int, or 'auto'
-        Categorical features,
-        type int represents index,
-        type str represents feature names (need to specify feature_name as well)
-        If 'auto' and data is pandas DataFrame, use pandas categorical columns
-    early_stopping_rounds: int
+    init_model : string or None, optional (default=None)
+        Filename of LightGBM model or Booster instance used for continue training.
+    feature_name : list of strings or 'auto', optional (default="auto")
+        Feature names.
+        If 'auto' and data is pandas DataFrame, data columns names are used.
+    categorical_feature : list of strings or int, or 'auto', optional (default="auto")
+        Categorical features.
+        If list of int, interpreted as indices.
+        If list of strings, interpreted as feature names (need to specify ``feature_name`` as well).
+        If 'auto' and data is pandas DataFrame, pandas categorical columns are used.
+    early_stopping_rounds: int or None, optional (default=None)
         Activates early stopping. CV error needs to decrease at least
-        every <early_stopping_rounds> round(s) to continue.
+        every ``early_stopping_rounds`` round(s) to continue.
         Last entry in evaluation history is the one from best iteration.
-    fpreproc : function
-        Preprocessing function that takes (dtrain, dtest, param)
+    fpreproc : callable or None, optional (default=None)
+        Preprocessing function that takes (dtrain, dtest, params)
         and returns transformed versions of those.
-    verbose_eval : bool, int, or None, default None
+    verbose_eval : bool, int, or None, optional (default=None)
         Whether to display the progress.
         If None, progress will be displayed when np.ndarray is returned.
-        If True, progress will be displayed at boosting stage.
-        If an integer is given,
-            progress will be displayed at every given `verbose_eval` boosting stage.
-    show_stdv : bool, default True
+        If True, progress will be displayed at every boosting stage.
+        If int, progress will be displayed at every given ``verbose_eval`` boosting stage.
+    show_stdv : bool, optional (default=True)
         Whether to display the standard deviation in progress.
-        Results are not affected, and always contains std.
-    seed : int
+        Results are not affected by this parameter, and always contains std.
+    seed : int, optional (default=0)
         Seed used to generate the folds (passed to numpy.random.seed).
-    callbacks : list of callback functions
+    callbacks : list of callables or None, optional (default=None)
         List of callback functions that are applied at each iteration.
-        See Callbacks in Python-API.md for more information.
+        See Callbacks in Python API for more information.
 
     Returns
     -------
-    evaluation history : list(string)
+    eval_hist : dict
+        Evaluation history.
+        The dictionary has the following format:
+        {'metric1-mean': [values], 'metric1-stdv': [values],
+        'metric2-mean': [values], 'metric1-stdv': [values],
+        ...}.
     """
     if not isinstance(train_set, Dataset):
         raise TypeError("Traninig only accepts Dataset object")
